import { ILinkedReferences } from "./linked-references.js";
/**
 * @public
 * Provides a way to handle cleanup of manually managed resources where there is not a single owner.
 */
export interface IReferenceCounted {
    claim(): void;
    release(): void;
    getIsDestroyed(): boolean;
    getLinkedReferences(): ILinkedReferences;
    /**
     * Callback will be called when the reference count hits 0. Useful for cleanup.
     */
    registerOnFreeListener(callback: () => void): void;
    /**
     * Remove the listener set by "registerOnFreeListener".
     */
    unregisterOnFreeListener(callback: () => void): void;
}
/**
 * @public
 * Provides a way to handle cleanup of manually managed resources where there is not a single owner.
 * NB The object is pre-claimed (ref count 1) on creation.
 */
export declare abstract class AReferenceCounted implements IReferenceCounted {
    /**
     * Take a claim on the object, preventing it from being destroyed. Once you're done with the object you should
     * call `release`.
     */
    claim(): void;
    /**
     * You should only call `release` if you called `claim` first, except for the object creator, who automatically stakes a claim.
     */
    release(): void;
    getIsDestroyed(): boolean;
    getLinkedReferences(): ILinkedReferences;
    registerOnFreeListener(callback: () => void): void;
    unregisterOnFreeListener(callback: () => void): void;
    /**
     * DO NOT CALL THIS DIRECTLY, CALL RELEASE.
     */
    protected onFree(): void;
    private references;
    private linkedReferences;
    private onFreeListener;
}
